/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* contact.cpp                                                             *
* Handles the contact list                                                *
**************************************************************************/


#include "contact.h"

struct CONTACT contactList[MAXCONTACT];
int  contactNum;
FILE *contactFile;

/*
** 
** Reads the contact list from a file
** Returns 0 of the contact file was found and -1 if not
*/
int contact_init(char *contactPath)
{
    char string[BUFSIZE];
    char *string2;
    char *tmp;
    int i;

    contactFile=fopen(contactPath,"r");
    if (!contactFile)
    {
        char tmp[BUFSIZE];
        sprintf(tmp, "Couldn't open %s.", contactPath);
        misc_log(tmp);

        contactNum = 0;

        return -1;
    }
    else
    {
        contactNum = -1;

        while (fgets(string, BUFSIZE,contactFile))
        {
            contactNum++;

            contactList[contactNum].status = STATUS_OFFLINE;
            
            string2 = string;
            //string2 = strdup(string);

            tmp = strchr(string2, ':');
            *tmp=0;
            contactList[contactNum].uin = atoi(string2);
            string2=tmp+1;
            tmp = strtok(string2, "\n");
            *(tmp-1)=0;
            strcpy(contactList[contactNum].nick, string2);
            string2=tmp+strlen("\n");

            contactList[contactNum].ip = inet_addr("0.0.0.0");
            contactList[contactNum].real_ip = inet_addr("0.0.0.0");
            contactList[contactNum].port = 0;

            if (contactList[contactNum].uin == 0)
                contactNum--;
        }

        fclose(contactFile);

	if(DEBUG)
        	for(i=contactNum;i--;i>-1) printf(" %s %d\n",contactList[i].nick,contactList[i].uin);

        return 0;
    }
}


/*
** 
** Return the number of entries in the contactList
*/
int contact_length()
{
    return contactNum + 1;
}


/*
** 
** Returns an entry in the contact list, or an entry with uin 0 on
** error
*/
struct CONTACT contact_returnItemNum(int num)
{
    if (num <= contactNum)
        return contactList[num];
    else
    {
        struct CONTACT contact;
        contact.uin = 0;
        
        return contact;
    }
}


/*
** 
** Convert uin to nick
*/
char *contact_uin2nick(unsigned long int uin)
{
    int i;
    char *tmp;

    for (i = 0; i <= contactNum; i++)
    {
        if (uin == contactList[i].uin)
            return contactList[i].nick;
    }
    //char tmp[BUFSIZE];

    tmp = (char *) malloc(BUFSIZE);
    sprintf(tmp, "%ld", uin);
    return tmp;
}


/*
** 
** Convert nick to uin
*/
unsigned long int contact_nick2uin(char *nick)
{
    int i;

    for (i = 0; i <= contactNum; i++)
    {
        if (!strcasecmp(nick, contactList[i].nick))
            return contactList[i].uin;
    }

    return(atoi(nick));
}


/*
** 
** Return entry which match uin, or entry with uin 0 on error
*/
struct CONTACT contact_returnContact(unsigned long int uin)
{
    int i;
    struct CONTACT contact;

    for (i = 0; i <= contactNum; i++)
    {
        if (uin == contactList[i].uin)
        {
            return contactList[i];
        }
    }

    contact.uin = 0;
    return contact;
}


/*
** 
** Update an entry
** Return 0 on success, -1 on error
*/
int contact_updateContact(struct CONTACT update_contact)
{
    int i;

    for (i = 0; i <= contactNum; i++)
    {
        if (update_contact.uin == contactList[i].uin)
        {
            contactList[i] = update_contact;
            return 0;
        }
    }

    return -1;
}


char *contact_status2str(long int status)
{
    char *string;

    switch (status) {
        case 0x0:
        // This MIGHT be DND.. 
        case 0x10013:
        case 0x10000:
        // This MIGHT be N/A..
        case 0x10005:
        case 0x20000:
        case 0x30000: {
            return "Available";
            break;
	}
        case 0x1:
        case 0x10001:
        case 0x20001:
        case 0x30001:
            return "Away";
            break;
        case 0x5:
        case 0x20005:
        case 0x30005:
            return "N/A";
            break;
        case 0x11:
        case 0x20011:
        case 0x30011:
            return "Occupied";
            break;
        case 0x13:
	case 0x20013:
        case 0x30013:
            return "DND";
            break;
        case 0x20:
        case 0x20020:
        case 0x30020:
            return "Free for chat";
            break;
        case 0x100:
            return "Invisible";
            break;
        case 0xFFFFFFFF:
            return "Offline";
        default:
            //char string[BUFSIZE];
	    string = (char *) malloc(BUFSIZE);
            sprintf(string, "Unknown(%ld)", status);
            return string;
            break;
    }

    return '\0';
}// contact_status2str






