/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* parse.cpp                                                               *
* Parse incomming packets                                                 *
**************************************************************************/


#include "parse.h"
#include "callbacks.h"

extern struct USER_CONFIGURATION user_config;
extern struct CALLBACK_FUNCTIONS callback;


/*
** Parse received packets and activate the callback function for it
** Returns -1 if something goes wrong
*/
int parse(struct ICQ_PACKET *packet)
{
    switch (packet->command)
    {
        case S_ACK:// Server acked a received package
            {
                if (DEBUG)
                    misc_log("Got ack.");

                break;
            }
        case S_LOGIN_REPLY:// Server acked out login packet
            {
                if (DEBUG)
                    misc_log("Got login reply.");

                send_ack(packet->sequence);
                user_config.status = user_config.defaultStatus;
                send_statusChange(user_config.status);

                send_login_1();
                send_contactList();

//                (callback.loggedIn)();
		loggedIn();
                break;
            }
        case S_USER_ONLINE:// Someone from the contact list is online
            {
                if (DEBUG)
                    misc_log("Got user online.");
                
                parse_S_USER_ONLINE(packet);
                break;
            }
        case S_USER_OFFLINE:// Someone logged out
            {
                if (DEBUG)
                    misc_log("Got user offline.");
                
                parse_S_USER_OFFLINE(packet);
                break;
            }
        case S_STATUS_UPDATE:// A user have changed status
            {
                if (DEBUG)
                    misc_log("Got user status change.");
                
                parse_S_STATUS_UPDATE(packet);
                break;
            }
        case S_GET_MESSAGE:// MSG sent while online
            {
                send_ack(packet->sequence);
                parse_S_GET_MESSAGE(packet, 1);
                break;
            }
        case S_RECEIVE_MESSAGE:// MSG send while offline
            {
                send_ack(packet->sequence);
                send_msgAck(packet->sequence);
                parse_S_GET_MESSAGE(packet, 0);
                break;
            }
	case S_USER_FOUND: // User matched our query
	    {
		send_ack(packet->sequence);
		parse_S_USER_FOUND(packet);
		break;
	    }
	case S_END_OF_SEARCH:// No more users matching, or too many found
	    {
		send_ack(packet->sequence);
		break;
	    }
        case S_GO_AWAY:// We've been idle,
        case S_DISCONNECTED:// We got disconnected?
            {
                contact_init(user_config.contactList);
//                (*callback.disconnected)();
		disconnected();
            }
        case S_REPLY_X1:// Unknown packet, sent during login
            {
                if (DEBUG)
                    misc_log("Got unknown login reply.");
                
                send_ack(packet->sequence);
                break;
            }
        case S_REPLY_X2:// Unknown packet, sent during login
            {
                if (DEBUG)
                    misc_log("Got unknown login reply.");
                
                send_ack(packet->sequence);
                break;
            }
        default:
            {
                if (DEBUG)
                {
                    char tmp[BUFSIZE];
                    sprintf(tmp, "Unknown packet, CMD: %04X - SEQ: %d", packet->command, packet->sequence);
                    misc_log(tmp);
                }

                send_ack(packet->sequence);
            }
    }

    //misc_printPacket(packet);
    return -1;
}


/*
** 
** User from contact list is online, find out who, and notify the UI
*/
void parse_S_USER_ONLINE(struct ICQ_PACKET *packet)
{
    struct CONTACT contact;
    unsigned long int *uin;
    unsigned long int *status;

    uin    = (unsigned long int *)(packet->data + 6);
    status = (unsigned long int *)(packet->data + 23);

    contact = contact_returnContact(*uin);

    contact.uin    = *uin;
    contact.status = *status;

    contact_updateContact(contact);

    send_ack(packet->sequence);

//    (*callback.updateContact)(*uin);
	updateContact(*uin);
}


/*
** 
** User from contact list is offline, find out who, and notify the UI
*/
void parse_S_USER_OFFLINE(struct ICQ_PACKET *packet)
{
    struct CONTACT contact;
    unsigned long int *uin;

    uin = (unsigned long int *)(packet->data + 6);

    contact = contact_returnContact(*uin);

    contact.uin    = *uin;
    contact.status = STATUS_OFFLINE;

    contact_updateContact(contact);

    send_ack(packet->sequence);

//    (*callback.updateContact)(*uin);
	updateContact(*uin);
}


/*
** 
** User from contact list has changed status, find out who, and notify the UI
*/
void parse_S_STATUS_UPDATE(struct ICQ_PACKET *packet)
{
    struct CONTACT contact;
    unsigned long int *uin;
    unsigned long int *status;

    uin    = (unsigned long int *)(packet->data + 6);
    status = (unsigned long int *)(packet->data + 10);

    contact = contact_returnContact(*uin);

    contact.uin    = *uin;
    contact.status = *status;

    contact_updateContact(contact);

    send_ack(packet->sequence);

//    (*callback.updateContact)(*uin);
	updateContact(*uin);
}


/*
** 
** Got a message packet, find out if it contains a message or an URL
*/
void parse_S_GET_MESSAGE(struct ICQ_PACKET *packet, int online)
{
    unsigned short int *type;
    
    if (online)
         type = (unsigned short int *)(packet->data + 10);
    else
        type = (unsigned short int *)(packet->data + 16);

    switch (*type)
    {
        case 0x0001:// Message
            {
                if (DEBUG)
                    misc_log("Got message.");
                
                parse_message(packet, online);
                break;
            }
        case 0x0004:// URL
            {
                if (DEBUG)
                    misc_log("Got URL.");
                
                parse_url(packet, online);
                break;
            }
        case 0x000C:// You've been added.
            {
                if (DEBUG)
                    misc_log("Got You've been added.");
                
                parse_added(packet, online);
                break;
            }
    }
}


/*
** 
** Got a message, translate it, and notify the UI
*/
void parse_message(struct ICQ_PACKET *packet, int online)
{
    unsigned long int *uin;
    unsigned int *msgLength;
    time_t t;
    struct MESSAGE messages;
    unsigned short int *year;
    unsigned char *month;
    unsigned char *day;
    unsigned char *hour;
    unsigned char *min;
    unsigned char *msg;
    struct tm *t2;

    uin =  (unsigned long int *)(packet->data + 6);

    if (online) {
        *msgLength = (packet->data + 12);
        msg = (packet->data + 14);

        t = time(NULL);
        t2   = localtime(&t);

        messages.online     = 1;
        messages.time.year  = t2->tm_year + 1900;
        messages.time.month = t2->tm_mon + 1;
        messages.time.day   = t2->tm_mday;
        messages.time.hour  = t2->tm_hour;
        messages.time.min   = t2->tm_min;
        messages.sender     = *uin;
        messages.length     = *msgLength;
        messages.message    = (char *)msg;

 //       (*callback.message)(&message);
	message(&messages);
    } else {
        year =      (unsigned short int *)(packet->data + 10);
        month =          (unsigned char *)(packet->data + 12);
        day =            (unsigned char *)(packet->data + 13);
        hour =           (unsigned char *)(packet->data + 14);
        min =            (unsigned char *)(packet->data + 15);
        msgLength =                       (packet->data + 18);
        msg =                             (packet->data + 20);

        messages.online     = 0;
        messages.time.year  = *year;
        messages.time.month = *month;
        messages.time.day   = *day;
        messages.time.hour  = *hour;
        messages.time.min   = *min;
        messages.sender     = *uin;
        messages.length     = *msgLength;
        messages.message    = (char *)msg;

 //       (*callback.message)(&message);
	message(&messages);
    }
}


/*
** 
** Got an URL, translate it, and notify the UI
*/
void parse_url(struct ICQ_PACKET *packet, int online)
{
    unsigned long int *uin;
    char tmp[2];
    char *desc;
    char *_url;
    time_t t;
    struct URL urls;
    struct tm *t2;

    uin  =  (unsigned long int *)(packet->data + 6);

    if (online)
    {
        unsigned char *msg =            (unsigned char *)(packet->data + 14);

        char *string;

        string = (char *)msg;
        if(DEBUG) printf("%s \n",string);
//        string = strdup((char *)msg);

        tmp[0] = 0xFE;
        tmp[1] = '\0';

        desc = strsep(&string, tmp);
//        _url  = strsep(&string, "");
	_url=string;
	_url++;

	if(DEBUG) printf("Desc: %s url: %s\n",desc,_url);

        t = time(NULL);
        t2   = localtime(&t);
        
        urls.online     = 1;
        urls.time.year  = t2->tm_year + 1900;
        urls.time.month = t2->tm_mon + 1;
        urls.time.day   = t2->tm_mday;
        urls.time.hour  = t2->tm_hour;
        urls.time.min   = t2->tm_min;
        urls.sender = *uin;
        urls.url    = (char *)_url;
        urls.desc   = (char *)desc;

 //       (*callback.url)(&url);
	url(&urls);
    }
    else
    {
        unsigned short int *year =      (unsigned short int *)(packet->data + 10);
        unsigned char *month =          (unsigned char *)(packet->data + 12);
        unsigned char *day =            (unsigned char *)(packet->data + 13);
        unsigned char *hour =           (unsigned char *)(packet->data + 14);
        unsigned char *min =            (unsigned char *)(packet->data + 15);
        unsigned char *msg =            (unsigned char *)(packet->data + 20);

        char *string;
        string = (char *) msg;   
        if(DEBUG) printf("%s \n",string);   

//        string = strdup((char *)msg);
        tmp[0] = 0xFE;
        tmp[1] = '\0';

        desc = strsep(&string, tmp);
//        _url  = strsep(&string, "");
        _url=string;
        _url++;
 
        if(DEBUG) printf("Desc: %s url: %s\n",desc,_url);

        urls.time.year  = *year;
        urls.time.month = *month;
        urls.time.day   = *day;
        urls.time.hour  = *hour;
        urls.time.min   = *min;
        urls.sender     = *uin;
        urls.url        = (char *)_url;
        urls.desc       = (char *)desc;

 //       (*callback.url)(&url);
	url(&urls);
    }
}


/*
** 
** You were added to a contact list, find out whos, and notify the UI
*/
void parse_added(struct ICQ_PACKET *packet, int online)
{
    char tmp[2];
    unsigned long int *uin;
    char *nick, *firstName, *lastName, *email, *auth, *string;
    time_t t;
    struct ADDED addeds;
    struct tm *t2;

    uin  =  (unsigned long int *)(packet->data + 6);

    if (online)
    {
        unsigned char *msg =            (unsigned char *)(packet->data + 14);

        string = (char *)msg;
//        string = strdup((char *)msg);
        tmp[0] = 0xFE;
        tmp[1] = '\0';

        nick          = strsep(&string, tmp);
        firstName     = strsep(&string, tmp);
        lastName      = strsep(&string, tmp);
        email         = strsep(&string, tmp);
        auth          = strsep(&string, "");

        t = time(NULL);
        t2   = localtime(&t);

        addeds.online     = 1;
        addeds.time.year  = t2->tm_year;
        addeds.time.month = t2->tm_mon + 1;
        addeds.time.day   = t2->tm_mday;
        addeds.time.hour  = t2->tm_hour;
        addeds.time.min   = t2->tm_min;
        addeds.sender     = *uin;
        addeds.nick       = (char *)nick;
        addeds.firstName  = (char *)firstName;
        addeds.lastName   = (char *)lastName;
        addeds.email      = (char *)email;

        if (auth[0] == '0')
            addeds.auth = 0;// You need authorization to add this user
        else
            addeds.auth = 1;// You don't need authorization to add this user

/*hoppar |ver callbackbiten och fixar namnaproblem*/        
 //       (*callback.added)(&added);
	added(&addeds); 
    }
    else
    {
        unsigned short int *year =      (unsigned short int *)(packet->data + 10);
        unsigned char *month =          (unsigned char *)(packet->data + 12);
        unsigned char *day =            (unsigned char *)(packet->data + 13);
        unsigned char *hour =           (unsigned char *)(packet->data + 14);
        unsigned char *min =            (unsigned char *)(packet->data + 15);
        unsigned char *msg =            (unsigned char *)(packet->data + 20);

        char *string;
        string = (char *)msg;
//        string = strdup((char *)msg);
        tmp[0] = 0xFE;
        tmp[1] = '\0';
        nick          = strsep(&string, tmp);
        firstName     = strsep(&string, tmp);
        lastName      = strsep(&string, tmp);
        email         = strsep(&string, tmp);
        auth          = strsep(&string, "");

        addeds.online     = 0;
        addeds.time.year  = *year;
        addeds.time.month = *month;
        addeds.time.day   = *day;
        addeds.time.hour  = *hour;
        addeds.time.min   = *min;
        addeds.sender     = *uin;
        addeds.nick       = (char *)nick;
        addeds.firstName  = (char *)firstName;
        addeds.lastName   = (char *)lastName;
        addeds.email      = (char *)email;

        if (auth[0] == '0')
            addeds.auth = 0;// You need authorization to add the user
        else
            addeds.auth = 1;// You don't need authorization to add this user
//hoppar |ver callback och l|ser ett namnproblem
//        (*callback.added)(&added);
	added(&addeds); 
    }
}


/*
**
** A user was found matching our latest search query
*/
void parse_S_USER_FOUND(struct ICQ_PACKET *packet)
{
    unsigned long int       *uin;
    unsigned short int      *nickLength;
    unsigned char           *nick;
    unsigned short int      *firstLength;
    unsigned char           *first;
    unsigned short int      *lastLength;
    unsigned char           *last;
    unsigned short int      *emailLength;
    unsigned char           *email;
    unsigned char           *auth;
    struct SEARCH_RESULT result;
    int pos = 8;

    uin =         (unsigned long int *)(packet->data + pos);     pos += 4;
    nickLength =  (unsigned short int *)(packet->data + pos);    pos += 2;
    nick =        (unsigned char *)(packet->data + pos);         pos += *nickLength;
    firstLength = (unsigned short int *)(packet->data + pos);    pos += 2;
    first =       (unsigned char *)(packet->data + pos);         pos += *firstLength;
    lastLength =  (unsigned short int *)(packet->data + pos);    pos += 2;
    last =        (unsigned char *)(packet->data + pos);         pos += *lastLength;
    emailLength = (unsigned short int *)(packet->data + pos);    pos += 2;
    email =       (unsigned char *)(packet->data + pos);         pos += *emailLength;
    auth =        (unsigned char *)(packet->data + pos);

    result.uin =       *uin;
    result.nick =      (char *)nick;
    result.firstName = (char *)first;
    result.lastName =  (char *)last;
    result.email =     (char *)email;
    if (auth[0] == '0')
        result.auth = 0;// You need authorization to add the user
    else
        result.auth = 1;// You don't need authorization to add this user

//    (*callback.searchResult)(&result);
	searchResult(&result); 
}
